@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.38';
+$VERSION = '1.39';
 
 sub ProcessJpeg2000Box($$$);
 sub ProcessJUMD($$$);
@@ -141,10 +141,13 @@ my %j2cMarker = (
         Authenticity Initiative) JUMBF (JPEG Universal Metadata Box Format) metdata
         is currently extracted from JPEG, PNG, TIFF-based (eg. TIFF, DNG),
         QuickTime-based (eg. MP4, MOV, HEIF, AVIF), RIFF-based (eg. WAV, AVI, WebP),
-        GIF files and ID3v2 metadata.  The suggested ExifTool command-line arguments
-        for reading C2PA metadata are C<-jumbf:all -G3 -b -j -u -struct>.  This
-        metadata may be deleted from writable JPEG, PNG, WebP, TIFF-based, and
-        QuickTime-based files by deleting the JUMBF group with C<-jumbf:all=>.
+        PDF, SVG and GIF files, and ID3v2 metadata.  The suggested ExifTool
+        command-line arguments for reading C2PA metadata are C<-jumbf:all -G3 -b -j
+        -u -struct>.  This metadata may be deleted from writable JPEG, PNG, WebP,
+        TIFF-based, and QuickTime-based files by deleting the JUMBF group with
+        C<-jumbf:all=>.  The C2PA JUMBF metadata may be extracted as a block via the
+        JUMBF tag.  See L<https://c2pa.org/specifications/> for the C2PA
+        specification.
     },
 #
 # NOTE: ONLY TAGS WITH "Format" DEFINED ARE EXTRACTED!
@@ -419,7 +422,7 @@ my %j2cMarker = (
         Flags => [ 'Binary', 'Protected' ],
         SubDirectory => { TagTable => 'Image::ExifTool::CBOR::Main' },
     },
-    bfdb => { # used in JUMBF (see  # (used when tag is renamed according to JUMDLabel)
+    bfdb => { # used in JUMBF
         Name => 'BinaryDataType',
         Notes => 'JUMBF, MIME type and optional file name',
         Format => 'undef',
@@ -748,6 +751,7 @@ my %j2cMarker = (
         },
         # seen:
         # cacb/cast/caas/cacl/casg/json-00110010800000aa00389b71
+        #   (also brob- but not yet tested)
         # 6579d6fbdba2446bb2ac1b82feeb89d1 - JPEG image
     },
     'label' => { Name => 'JUMDLabel' },
@@ -821,6 +825,7 @@ sub ProcessJUMD($$$)
             $name =~ tr/-_a-zA-Z0-9//dc;    # remove other illegal characters
             $name =~ s/__/_/;               # collapse double underlines
             $name = ucfirst $name;          # capitalize first letter
+            $name =~ s/C2pa/C2PA/;          # capitalize C2PA
             $name = "Tag$name" if length($name) < 2; # must at least 2 characters long
             $$et{JUMBFLabel} = $name;
         }
@@ -1017,11 +1022,23 @@ sub ProcessJpeg2000Box($$$)
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $base = $$dirInfo{Base} || 0;
     my $outfile = $$dirInfo{OutFile};
+    my $dirName = $$dirInfo{DirName} || '';
     my $dirEnd = $dirStart + $dirLen;
     my ($err, $outBuff, $verbose, $doColour, $hash, $raf);
 
-    # read from RAF unless reading from buffer
-    $raf = $$dirInfo{RAF} unless $dataPt;
+    if ($dataPt) {
+        # save C2PA JUMBF as a block if requested
+        if ($dirName eq 'JUMBF' and $$et{REQ_TAG_LOOKUP}{jumbf} and not $$dirInfo{NoBlockSave}) {
+            if ($dirStart or $dirLen ne length($$dataPt)) {
+                my $dat = substr($$dataPt, $dirStart, $dirLen);
+                $et->FoundTag(JUMBF => \$dat);
+            } else {
+                $et->FoundTag(JUMBF => $dataPt);
+            }
+        }
+    } else {
+        $raf = $$dirInfo{RAF};  # read from RAF
+    }
 
     if ($outfile) {
         unless ($raf) {
@@ -1030,7 +1047,7 @@ sub ProcessJpeg2000Box($$$)
             $outfile = \$outBuff;
         }
         # determine if we will be writing colr box
-        if ($$dirInfo{DirName} and $$dirInfo{DirName} eq 'JP2Header') {
+        if ($dirName eq 'JP2Header') {
             $doColour = 2 if defined $et->GetNewValue('ColorSpecMethod') or $et->GetNewValue('ICC_Profile') or
                              defined $et->GetNewValue('ColorSpecPrecedence') or defined $et->GetNewValue('ColorSpace') or
                              defined $et->GetNewValue('ColorSpecApproximation') or defined $et->GetNewValue('ColorSpecData');
@@ -1038,7 +1055,7 @@ sub ProcessJpeg2000Box($$$)
     } else {
         # (must not set verbose flag when writing!)
         $verbose = $$et{OPTIONS}{Verbose};
-        $et->VerboseDir($$dirInfo{DirName}) if $verbose;
+        $et->VerboseDir($dirName) if $verbose;
         # do hash if requested, but only for top-level image data
         $hash = $$et{ImageDataHash} if $raf;
     }
@@ -1187,7 +1204,7 @@ sub ProcessJpeg2000Box($$$)
         # create new tag for JUMBF data values with name corresponding to JUMBFLabel
         if ($tagInfo and $$et{JUMBFLabel} and (not $$tagInfo{SubDirectory} or $$tagInfo{BlockExtract})) {
             $tagInfo = { %$tagInfo, Name => $$et{JUMBFLabel} . ($$tagInfo{JUMBF_Suffix} || '') };
-            delete $$tagInfo{Description};
+            ($$tagInfo{Description} = Image::ExifTool::MakeDescription($$tagInfo{Name})) =~ s/C2 PA/C2PA/;
             AddTagToTable($tagTablePtr, '_JUMBF_' . $$et{JUMBFLabel}, $tagInfo);
             delete $$tagInfo{Protected}; # (must do this so -j -b returns JUMBF binary data)
             $$tagInfo{TagID} = $boxID;
@@ -1491,6 +1508,28 @@ sub ProcessJXLCodestream($$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Read/write meta information from a C2PA/JUMBF file
+# Inputs: 0) ExifTool object reference, 1) dirInfo reference
+# Returns: 1 on success, 0 if this wasn't a valid JUMBF file
+sub ProcessJUMBF($$)
+{
+    my ($et, $dirInfo) = @_;
+    my $raf = $$dirInfo{RAF};
+    my $hdr;
+
+    # check to be sure this is a valid JPG2000 file
+    return 0 unless $raf->Read($hdr,20) == 20 and $raf->Seek(0,0);
+    return 0 unless $hdr =~ /^.{4}jumb\0.{3}jumd(.{4})/;
+    $et->SetFileType($1 eq 'c2pa' ? 'C2PA' : 'JUMBF');
+    my %dirInfo = (
+        RAF => $raf,
+        DirName => 'JUMBF',
+    );
+    my $tagTablePtr = GetTagTable('Image::ExifTool::Jpeg2000::Main');
+    return $et->ProcessDirectory(\%dirInfo, $tagTablePtr);
+}
+
 #------------------------------------------------------------------------------
 # Read/write meta information from a JPEG 2000 image
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference
