@@ -14,6 +14,7 @@
 #               Sep. 16/08 - P. Harvey Improve timezone testing
 #               Jul. 14/10 - P. Harvey Added writeInfo()
 #               Jan. 06/12 - P. Harvey Patched MirBSD leap second "feature"
+#               Jun. 08/21 - PH Patched float compare to fix quadmath test failure
 #------------------------------------------------------------------------------
 
 package t::TestLib;
@@ -24,16 +25,18 @@ require Exporter;
 use Image::ExifTool qw(ImageInfo);
 
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION = '1.21';
+$VERSION = '1.23';
 @ISA = qw(Exporter);
-@EXPORT = qw(check writeCheck writeInfo testCompare binaryCompare testVerbose);
+@EXPORT = qw(check writeCheck writeInfo testCompare binaryCompare testVerbose notOK done);
 
 my $noTimeLocal;
+my $rtnCode = 0;
 
 sub nearEnough($$);
 sub nearTime($$$$);
 sub formatValue($);
-sub writeInfo($$;$$);
+sub writeInfo($$;$$$);
+sub notOK();
 
 #------------------------------------------------------------------------------
 # Compare 2 binary files
@@ -78,14 +81,28 @@ sub testCompare($$$;$)
             $success = 1;
             my ($line1, $line2);
             my $linenum = 0;
+            my $skip = 0;
             for (;;) {
-                $line1 = <FILE1>;
+                $line1 = <FILE1> unless $skip == 1;
                 last unless defined $line1;
                 ++$linenum;
-                $line2 = <FILE2>;
+                $line2 = <FILE2> unless $skip == 2;
+                $skip = 0;
                 if (defined $line2) {
                     next if $line1 eq $line2;
                     next if nearEnough($line1, $line2);
+                    # ignore IPTCDigest warning if Digest::MD5 isn't available
+                    if ($line1 =~ /Warning: IPTCDigest is not current/ and
+                        not eval 'require Digest::MD5')
+                    {
+                        $skip = 2; 
+                        next;
+                    } elsif ($line2 =~ /Warning: IPTCDigest is not current/ and
+                        not eval 'require Digest::MD5')
+                    {
+                        $skip = 1;
+                        next;
+                    }
                 }
                 $success = 0;
                 last;
@@ -137,7 +154,7 @@ sub nearEnough($$)
 
     # allow CurrentIPTCDigest to be zero if Digest::MD5 isn't installed
     return 1 if $line1 =~ /Current IPTC Digest/ and
-                $line2 =~ /Current IPTC Digest: 0{32}/ and
+                $line2 =~ /Current IPTC Digest: (0|#){32}/ and
                 not eval 'require Digest::MD5';
 
     # analyze every token in the line, and allow rounding
@@ -186,6 +203,10 @@ sub nearEnough($$)
             $tok1 .= ' ' . $toks1[$i];      # add time to give date/time value
             $tok2 .= ' ' . $toks2[$i];
             last unless nearTime($tok1, $tok2, $line1, $line2);
+        # handle floating point numbers filtered by ExifTool test 29
+        } elsif ($tok1 =~ s/(\.#)#*(e[-+]\#+)?/$1/g or $tok2 =~ s/(\.#)#*(e[-+]\#+)?/$1/g) {
+            $tok2 =~ s/(\.#)#*(e[-+]\#+)?/$1/g;
+            last if $tok1 ne $tok2;
         } else {
             # check to see if both tokens are floating point numbers (with decimal points!)
             if ($tok1 =~ s/([^\d.]+)$//) {  # remove trailing units
@@ -200,8 +221,7 @@ sub nearEnough($$)
                 last unless $tok2 =~ s/^'//;
             }
             last unless Image::ExifTool::IsFloat($tok1) and
-                        Image::ExifTool::IsFloat($tok2) and
-                        $tok1 =~ /\./ and $tok2 =~ /\./;
+                        Image::ExifTool::IsFloat($tok2);
             last if $tok1 == 0 or $tok2 == 0;
             # numbers are bad if not the same to 5 significant figures
             if (abs(($tok1-$tok2)/($tok1+$tok2)) > 1e-5) {
@@ -310,8 +330,12 @@ sub check($$$;$$$)
     # get a list of found tags
     my @tags;
     if ($exifTool) {
-        # sort tags by group to make it a bit prettier
-        @tags = $exifTool->GetTagList($info, 'Group0');
+        if ($$exifTool{NO_SORT}) {
+            @tags = $exifTool->GetFoundTags();
+        } else {
+            # sort tags by group to make it a bit prettier
+            @tags = $exifTool->GetTagList($info, 'Group0');
+        }
     } else {
         @tags = sort keys %$info;
     }
@@ -346,14 +370,15 @@ sub check($$$;$$$)
 #         1) test name, 2) test number, 3) optional source file name,
 #         4) true to only check tags which were written (or list ref for tags to check)
 #         5) flag set if nothing is expected to change in the output file
+#         6) true to ignore warnings
 # Returns: 1 if check passed
-sub writeCheck($$$;$$$)
+sub writeCheck($$$;$$$$)
 {
-    my ($writeInfo, $testname, $testnum, $srcfile, $onlyWritten, $same) = @_;
+    my ($writeInfo, $testname, $testnum, $srcfile, $onlyWritten, $same, $ignore) = @_;
     $srcfile or $srcfile = "t/images/$testname.jpg";
     my ($ext) = ($srcfile =~ /\.(.+?)$/);
     my $testfile = "t/${testname}_${testnum}_failed.$ext";
-    my $exifTool = new Image::ExifTool;
+    my $exifTool = Image::ExifTool->new;
     my @tags;
     if (ref $onlyWritten eq 'ARRAY') {
         @tags = @$onlyWritten;
@@ -364,7 +389,7 @@ sub writeCheck($$$;$$$)
         push @tags, $$_[0] if $onlyWritten;
     }
     unlink $testfile;
-    my $ok = writeInfo($exifTool, $srcfile, $testfile, $same);
+    my $ok = writeInfo($exifTool, $srcfile, $testfile, $same, $ignore);
     my $info = $exifTool->ImageInfo($testfile,{Duplicates=>1,Unknown=>1},@tags);
     my $rtnVal = check($exifTool, $info, $testname, $testnum);
     return 0 unless $ok and $rtnVal;
@@ -375,10 +400,11 @@ sub writeCheck($$$;$$$)
 #------------------------------------------------------------------------------
 # Call Image::ExifTool::WriteInfo with error checking
 # Inputs: 0) ExifTool ref, 1) src file, 2) dst file, 3) true if nothing should change
+#         4) true to ignore warnings
 # Return: true on success
-sub writeInfo($$;$$)
+sub writeInfo($$;$$$)
 {
-    my ($exifTool, $src, $dst, $same) = @_;
+    my ($exifTool, $src, $dst, $same, $ignore) = @_;
     # erase temporary file created by WriteInfo() if no destination file is given
     # (may be left over from previous crashed tests)
     unlink "${src}_exiftool_tmp" if not defined $dst and not ref $src;
@@ -387,6 +413,7 @@ sub writeInfo($$;$$)
     $err .= "  Error: WriteInfo() returned $result\n" if $result != ($same ? 2 : 1);
     my $info = $exifTool->GetInfo('Warning', 'Error');
     foreach (sort keys %$info) {
+        next if $ignore and $_ =~ /^Warning/;
         my $tag = Image::ExifTool::GetTagName($_);
         $err .= "  $tag: $$info{$_}\n";
     }
@@ -419,9 +446,24 @@ sub testVerbose($$$$)
     }
     $\ = $oldSep;       # restore output line separator
     close(TESTFILE);
+    close(TMPFILE);
     unlink("$testfile.tmp");
     return testCompare("$testfile.out","$testfile.failed",$testnum);
 }
 
+#------------------------------------------------------------------------------
+# One of the tests failed
+sub notOK()
+{
+    print 'not ';
+    $rtnCode = 1;
+}
+
+#------------------------------------------------------------------------------
+# Done tests and exit
+sub done()
+{
+    exit $rtnCode;
+}
 
 1; #end
